'''
rquote

A stock history data api and related tools

Copyright (c) 2021 Roi ZHAO

'''

from .main import get_price, get_stock_concepts, get_concept_stocks, get_bk_stocks
from .main import get_all_concepts, get_all_industries
from .main import get_cn_stock_list, get_hk_stocks_hsi, get_hk_stocks_ggt
from .main import get_cn_future_list, get_us_stocks_biggest, get_cn_fund_list
from .utils import WebUtils, BasicFactors
from .plots import PlotUtils



# -*- coding: utf-8 -*-

import json
import time
import re
import base64
import pandas as pd
from .utils import WebUtils, hget, logger


def make_tgts(mkts=['ch', 'hk', 'us', 'fund', 'future'], money_min=2e8) -> []:
    cands = []


def get_cn_stock_list(money_min=2e8):
    '''
    Return sorted stock list ordered by latest amount of money, cut at `money_min`
    item in returned list are [code, name, change, amount, mktcap]
    '''
    a = hget(
        base64.b64decode('aHR0cDovLzM4LnB1c2gyLmVhc3Rtb25leS5jb20vYXBpL3F0L2Ns'+
            'aXN0L2dldD9jYj1qUXVlcnkxMTI0MDk0NTg3NjE4NDQzNzQ4MDFfMTYyNzI4ODQ4O'+
            'Tk2MSZwbj0xJnB6PTEwMDAwJnBvPTEmbnA9MSZ1dD1iZDFkOWRkYjA0MDg5NzAwY2'+
            'Y5YzI3ZjZmNzQyNjI4MSZmbHR0PTImaW52dD0yJmZpZD1mNiZmcz1tOjArdDo2LG0'+
            '6MCt0OjgwLG06MSt0OjIsbToxK3Q6MjMmZmllbGRzPWYxMixmMTQsZjMsZjYsZjIxJl89'
            ).decode() + str(int(time.time()*1e3))
    )
    if a:
        a = json.loads(a.text.split(
            'jQuery112409458761844374801_1627288489961(')[1][:-2])

    # cdir = os.path.dirname(__file__)
    # with open(os.path.join(cdir, 'ranka'), 'wb') as f:
    #     f.write(a.content)
    # a = pd.read_excel(os.path.join(cdir, 'ranka'), header=1)
    # os.remove(os.path.join(cdir, 'ranka'))
    # a.columns = ['code', 'name', 'close', 'p_change', 'change', '_', '_', '_',
    #              'money', 'open', 'yest_close', 'high', 'low']
    # a = a[a.money > money_min]
    a = [ ['sh'+i['f12'] if i['f12'][0]=='6' else 'sz'+i['f12'],
         i['f14'], i['f3'], i['f6'], i['f21']] for i in a['data']['diff']
        if i['f6']!='-' and float(i['f6']) > money_min]
    #cands=[(i.code,i.name) for i in a[['code','name']].itertuples()]
    return a


# def get_hk_stocks_hotest80():
#     a = hget(
#         'http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php' +
#         '/Market_Center.getHKStockData?page=1&num=80&sort=amount&asc=0&node=' +
#         'qbgg_hk&_s_r_a=sort').text
#     if a:
#         a = [['hk'+i['symbol'], i['name'], i['changepercent'], i['amount'],
#             i['market_value']] for i in json.loads(a)]
#     return a


def get_us_stocks_biggest(k=60):
    # return list of [symbol, name, price, volume, mktcap, pe]
    uscands = []
    a = hget(
        "https://stock.finance.sina.com.cn/usstock/api/jsonp.php/IO.XSRV2."+
        "CallbackList['f0j3ltzVzdo2Fo4p']/US_CategoryService.getList?page=1"+
        "&num=60&sort=&asc=0&market=&id=", headers=WebUtils.headers()).text
    if a:
        uslist = json.loads(a.split('(',1)[1][:-2])['data']
        # Warning: symbol not fitted
        uscands = [('us' + i['symbol'], i['name'], i['price'], i['volume'],
            i['mktcap']) for i in uslist]
    return uscands


def get_cn_fund_list():
    '''
    Return sorted etf list (ordered by latest amount of money),
        of [code, name, change, amount, price]
    '''
    a = hget(base64.b64decode('aHR0cDovL3ZpcC5zdG9jay5maW5hbmNlLnNpbmEuY29tL'+
        'mNuL3F1b3Rlc19zZXJ2aWNlL2FwaS9qc29ucC5waHAvSU8uWFNSVjIuQ2FsbGJhY2tMaX'+
        'N0WydrMldhekswNk5Rd2xoeVh2J10vTWFya2V0X0NlbnRlci5nZXRIUU5vZGVEYXRhU2l'+
        'tcGxlP3BhZ2U9MSZudW09MTAwMCZzb3J0PWFtb3VudCZhc2M9MCZub2RlPWV0Zl9ocV9m'+
        'dW5kJiU1Qm9iamVjdCUyMEhUTUxEaXZFbGVtZW50JTVEPXhtNGkw').decode()).text
    if a:
        fundcands = [[i['symbol'], i['name'], i['changepercent'], i['amount'], i['trade']]
                     for i in json.loads(a.split('k2WazK06NQwlhyXv')[1][3:-2])]
    return fundcands


def get_cn_future_list():
    '''
    Return cn future id list, with prefix of `fu`
    e.g. ['fuSC2109',
          'fuRB2110',
          'fuHC2110',
          'fuFU2109',
          ...]
    '''
    a = hget('https://finance.sina.com.cn/futuremarket/').text
    if a:
        futurelist_active = [
            'fu' + i for i in re.findall(r'quotes/(.*?\d+).shtml', a)]
    return futurelist_active


def get_price(i, sdate='', edate='', freq='day', days=320, fq='qfq',
              dd=None) -> (str, str, pd.DataFrame):
    '''
    Args:
        sdate: start date
        edate: end date
        dd: data dictionary, any local cache with get/put methods
        days: day length of fetching, overwriting sdate
        fq: qfq for non
    '''
    if dd is not None:
        a = dd.get(i)
        if a:
            n, d = a
            logger.debug('loading price from dd {}'.format(i))
            return i, n, d
    logger.debug('fetching price of {}'.format(i))
    qtimg_stock = 'http://web.ifzq.gtimg.cn/appstock/app/newfqkline/get?param=' + \
            '{},{},{},{},{},{}'
    qtimg_stock_hk = 'http://web.ifzq.gtimg.cn/appstock/app/hkfqkline/get?' + \
            'param={},{},{},{},{},{}'
    qtimg_stock_us = 'http://web.ifzq.gtimg.cn/appstock/app/usfqkline/get?' + \
            'param={},{},{},{},{},{}'
    sina_future_d = 'https://stock2.finance.sina.com.cn/futures/api/jsonp.php/' + \
            'var%20t1nf_{}=/InnerFuturesNewService.getDailyKLine?symbol={}'
    sina_btc = 'https://quotes.sina.cn/fx/api/openapi.php/BtcService.getDayKLine?' + \
        'symbol=btcbtcusd'

    # sina_future_d.format('FB0','FB0')

    if i[:2] == 'BK':
        try:
            a = hget(base64.b64decode('aHR0cDovL3B1c2gyaGlzLmVhc3' +
                               'Rtb25leS5jb20vYXBpL3F0L3N0b2NrL2tsaW5lL2dldD9jYj1qUX' +
                               'VlcnkxMTI0MDIyNTY2NDQ1ODczNzY2OTcyXzE2MTc4NjQ1NjgxMz' +
                               'Emc2VjaWQ9OTAu').decode() + i +
                          '&fields1=f1%2Cf2%2Cf3%2Cf4%2Cf5' +
                          '&fields2=f51%2Cf52%2Cf53%2Cf54%2Cf55%2Cf56%2Cf57%2Cf58' +
                          '&klt=101&fqt=0&beg=19900101&end=20990101&_=1',
                          headers=WebUtils.headers())
            if not a:
                logger.warning('{} hget failed: {}'.format(i, a))
                return i, 'None', pd.DataFrame([])
            a = json.loads(a.text.split(
                'jQuery1124022566445873766972_1617864568131(')[1][:-2])
            if not a['data']:
                logger.warning('{} data empty: {}'.format(i, a))
                return i, 'None', pd.DataFrame([])
            name = a['data']['name']
            d = pd.DataFrame([i.split(',') for i in a['data']['klines']], columns=[
                             'date', 'open', 'close', 'high', 'low', 'vol', 'money', 'p'])
            d = d.set_index(['date']).astype(float)
            # d.index = pd.DatetimeIndex(d.index)
            return i, name, d
        except Exception as e:
            logger.warning('error fetching {}, err: {}'.format(i, e))
            return i, 'None', pd.DataFrame([])

    if i[:2] == 'fu':
        try:
            ix = i[2:] if i[-1]=='0' else i[2:-4]
            if ix == 'btc' or ix == 'BTC':
                url = sina_btc
                d = json.loads(hget(url).text)['result']['data'].split('|')
                d = pd.DataFrame([i.split(',') for i in d], 
                                 columns=['date', 'open', 'high', 'low', 'close', 'vol', 'amount'])
                return i, 'BTC', d
            else:
                d = pd.DataFrame(json.loads(hget(sina_future_d.format(
                        ix, ix)).text.split('(')[1][:-2]))
                d.columns = ['date', 'open', 'high', 'low', 'close', 'vol', 'p', 's']
            d = d.set_index(['date']).astype(float)
            # d.index = pd.DatetimeIndex(d.index)
            return i, ix, d
        except Exception as e:
            logger.warning('error get price {}, err {}'.format(i[2:-4], e))
            return i, 'None', pd.DataFrame([])

    if i[0] in ['0', '1', '3', '5', '6']:
        i = 'sh'+i if i[0] in ['5', '6'] else 'sz'+i
    if i[:2] in ['sh', 'sz']:
        url = qtimg_stock.format(i, freq, sdate, edate, days, fq)
    elif i[:2] == 'hk':
        url = qtimg_stock_hk.format(i, freq, sdate, edate, days, fq)
    elif i[:2] == 'us':
        url = qtimg_stock_us.format(i, freq, sdate, edate, days, fq)
    else:
        raise ValueError('target market not supported')
    a = hget(url)
    #a = json.loads(a.text.replace('kline_dayqfq=', ''))['data'][i]
    a = json.loads(a.text)['data'][i]
    name = ''
    try:
        for tkt in ['day', 'qfqday', 'hfqday', 'week', 'qfqweek', 'hfqweek',
                    'month', 'qfqmonth', 'hfqmonth']:
            if tkt in a:
                tk = tkt
                break
        b = pd.DataFrame([j[:6] for j in a[tk]],
                         columns=['date',
                                  'open',
                                  'close',
                                  'high',
                                  'low',
                                  'vol']).set_index(['date']).astype(float)
        if 'qt' in a:
            name = a['qt'][i][1]
    except Exception as e:
        logger.warning('error fetching {}, err: {}'.format(i, e))
    return i, name, b


def get_price_longer(i, l=2, dd={}):
    # default get price 320 day, l as years
    _, name, a = get_price(i, dd=dd)
    d1 = a.index.format()[0]
    for y in range(1, l):
        d0 = str(int(d1[:4]) - 1) + d1[4:]
        a = pd.concat((get_price(i, d0, d1)[2], a), 0).drop_duplicates()
        d1 = d0
    return i, name, a


def get_tick(tgts=[]):
    '''
    Get quotes of a tick
    tgt list format:
        us stocks like gb_symbol, e.g. gb_aapl, gb_goog
    Return list of dict of given symbols for current timestamp
    '''
    if not tgts:
        return []
    sina_tick = 'https://hq.sinajs.cn/?list='
    head_row = ['name', 'price', 'price_change_rate', 'timesec',
        'price_change', '_', '_', '_', '_', '_', 'volume', '_', '_',
         '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_', '_',
         '_', 'last_close', '_', '_', '_', 'turnover', '_', '_', '_', '_']

    if type(tgts) == list:
        tgts = ['gb_' + i.lower() for i in tgts]
    elif type(tgts) == str:
        tgts = ['gb_' + tgts]
    else:
        raise ValueError('tgt should be list or str, e.g. APPL,')

    a = hget(sina_tick + ','.join(tgts))
    if not a:
        logger.warning('hget failed {}'.format(tgts))
        return []

    try:
        dat = [i.split('"')[1].split(',') for i in a.text.split(';\n') if ',' in i]
        dat_trim = [{k:i[j] for j,k in enumerate(head_row) if k!='_'} for i in dat]
    except Exception as e:
        logger.warming('data not complete, check tgt be code str or list without'+
            ' prefix, your given: {}'.format(tgts))
    return dat_trim


def get_stock_concepts(i) -> []:
    '''
    Return concept id(start with `BK`) list of a stock, from eastmoney
    '''
    f10url = base64.b64decode('aHR0cDovL2YxMC5lYXN0bW9uZXkuY29tLy9Db3JlQ29uY2V' +
                              'wdGlvbi9Db3JlQ29uY2VwdGlvbkFqYXg/Y29kZT0=').decode()
    #drop_cons = ['融资融券', '创业板综', '深股通', '沪股通', '深成500', '长江三角']
    #drop_tails = ['板块', '概念', '0_', '成份', '重仓']
    url = f10url + i
    try:
        concepts = json.loads(hget(url).text)[
            'hxtc'][0]['ydnr'].split()
    except Exception as e:
        logger.error(str(e))
        concepts = ['']
    #concepts = [i for i in concepts if i not in drop_cons]
    #concepts = [i for i in concepts if i[-2:] not in drop_tails]
    #concepts = [i for i in concepts if '股' not in i]
    return concepts


def get_concept_stocks(bkid, dc=None):
    '''
    Return stocks of input bkid, e.g. BK0420, BK0900
    dc : dictionary of concepts, local cache with get/put
    '''
    if dc is not None:
        a = dc.get(bkid)
        if a:
            return a
    bkid = bkid if isinstance(bkid, str) else 'BK' + str(bkid).zfill(4)
    a = hget(
        base64.b64decode('aHR0cDovL3B1c2gyLmVhc3Rtb25leS5jb20vYXBpL3F0L2NsaXN0' +
                         'L2dldD9jYj1qUXVlcnkxMTIzMDQwNTcwNTM4NTY5NDcwMTA1XzE2MTgwNDc5OTA2O' +
                         'TAmZmlkPWY2MiZwbz0xJnB6PTUwMCZwbj0xJm5wPTEmZmx0dD0yJmludnQ9MiZmcz' +
                         '1iJTNB').decode() +
        bkid +
        '&fields=f3%2Cf6%2Cf12%2Cf14%2Cf21').text
    a = json.loads(
        a.split('jQuery1123040570538569470105_1618047990690(')[1][:-2])['data']['diff']
    logger.debug('get fresh conc {}'.format(bkid))
    a = [ ['sh'+i['f12'] if i['f12'][0]=='6' else 'sz'+i['f12'],
         i['f14'], i['f3'], i['f6'], i['f21']] for i in a]
    return a


def _east_list_fmt(burl, api_name):
    '''
    formatter of eastmoney api
    Return list of list:
        [sid, name, rise, amount, mkt]
    '''
    a = hget(base64.b64decode(burl).decode() +
        str(int(time.time()*1e3)))
    if a:
        a = a.text
    else:
        return
    a = json.loads(a.split(api_name+'(')[1][:-2])['data']['diff']
    a = [ [i['f12'],i['f14'], i['f3'], i['f6'], i['f20']] for i in a]
    return a


def get_all_industries():
    '''
    Return sorted industry item list ordered by latest amount of money,
    item in returned list are [code, name, change, amount, price]
    '''
    a = _east_list_fmt('aHR0cHM6Ly84Ny5wdXNoMi5lYXN0bW9uZXkuY29tL2FwaS9xdC9jbGl'+
        'zdC9nZXQ/Y2I9alF1ZXJ5MTEyNDAzNzExNzU2NTU3MTk3MTM0NV8xNjI3MDQ3MTg4NTk5'+
        'JnBuPTEmcHo9MTAwJnBvPTEmbnA9MSZ1dD1iZDFkOWRkYjA0MDg5NzAwY2Y5YzI3ZjZmN'+
        'zQyNjI4MSZmbHR0PTImaW52dD0yJmZpZD1mMyZmcz1tOjkwK3Q6MitmOiE1MCZmaWVsZH'+
        'M9ZjMsZjYsZjEyLGYxNCxmMjAsZjEwNCxmMTA1Jl89',
        'jQuery1124037117565571971345_1627047188599')
    logger.debug('get industries {}'.format(len(a)))
    return a


def get_all_concepts():
    '''
    Return sorted concept item list ordered by latest amount of money,
    item in returned list are [code, name, change, amount, price]
    '''
    a = _east_list_fmt('aHR0cHM6Ly8yMi5wdXNoMi5lYXN0bW9uZXkuY29tL2FwaS9xdC9jbGl'+
        'zdC9nZXQ/Y2I9alF1ZXJ5MTEyNDA3MzI5ODQxOTMwNzY4OTc5XzE2MjcxMDk0NjA2MzMm'+
        'cG49MSZwej00MDAmcG89MSZucD0xJnV0PWJkMWQ5ZGRiMDQwODk3MDBjZjljMjdmNmY3N'+
        'DI2MjgxJmZsdHQ9MiZpbnZ0PTImZmlkPWYzJmZzPW06OTArdDozK2Y6ITUwJmZpZWxkcz'+
        '1mMyxmNixmMTIsZjE0LGYyMCxmMTA0LGYxMDUmXz0='
        ,'jQuery112407329841930768979_1627109460633')
    logger.debug('get concepts {}'.format(len(a)))
    return a


def get_bk_stocks(bkid):
    '''
    Return stock item list of given bk id,
    item in returned list are [code, name, change, amount, price]
    '''
    url = base64.b64decode('aHR0cDovLzgyLnB1c2gyLmVhc3Rtb25leS5jb20vYXBpL3F0L2'+
        'NsaXN0L2dldD9jYj1qUXVlcnkxMTI0MDQ4Njk5NjMwMDk1MTM3NzE0XzE2Mjc0Nzc0OTU'+
        'wNjQmcG49MSZwej0yMDAwJnBvPTAmbnA9MSZ1dD1iZDFkOWRkYjA0MDg5NzAwY2Y5YzI3'+
        'ZjZmNzQyNjI4MSZmbHR0PTImaW52dD0yJmZpZD1mNiZmcz1iOg==').decode()+ \
        bkid + '+f:!50&fields=f3,f6,f12,f14,f20&_='
    a = _east_list_fmt(base64.b64encode(bytes(url, encoding='utf-8')),
        'jQuery1124048699630095137714_1627477495064')
    logger.debug('get bk stocks {}'.format(len(a)))
    return a


def get_industry_stocks(bkid):
    '''
    Return sorted industry item list ordered by latest amount of money,
    item in returned list are [code, name, change, amount, price]
    '''
    url = base64.b64decode('aHR0cHM6Ly82Mi5wdXNoMi5lYXN0bW9uZXkuY29tL2FwaS9xdC'+
        '9jbGlzdC9nZXQ/Y2I9alF1ZXJ5MTEyNDA4Mzc4MjAwMDc0NDQ0MzA5XzE2Mjc4MjQ2MDM'+
        '1NjImcG49MSZwej0yMDAwJnBvPTAmbnA9MSZ1dD1iZDFkOWRkYjA0MDg5NzAwY2Y5YzI3'+
        'ZjZmNzQyNjI4MSZmbHR0PTImaW52dD0yJmZpZD1mNiZmcz1iOg==').decode()+ \
        bkid + '+f:!50&fields=f3,f6,f12,f14,f20&_='
    a = _east_list_fmt(base64.b64encode(bytes(url, encoding='utf-8')),
        'jQuery112408378200074444309_1627824603562')
    logger.debug('get industry stocks {}'.format(len(a)))
    return a


def get_hk_stocks_ggt():
    '''
    Return sorted stock item list in GangGuTong, ordered by amount of money,
    item in returned list are [code, name, change, amount, price]
    '''
    a = _east_list_fmt('aHR0cHM6Ly8yLnB1c2gyLmVhc3Rtb25leS5jb20vYXBpL3F0L2NsaX'+
        'N0L2dldD9jYj1qUXVlcnkxMTI0MDI0MzYyMzA4OTA2NjE1MDgyXzE2MjgyNTg5MzEyMjQ'+
        'mcG49MSZwej0xMDAwJnBvPTAmbnA9MSZ1dD1iZDFkOWRkYjA0MDg5NzAwY2Y5YzI3ZjZm'+
        'NzQyNjI4MSZmbHR0PTImaW52dD0yJmZpZD1mNiZmcz1iOkRMTUswMTQ2LGI6RExNSzAxN'+
        'DQmZmllbGRzPWYzLGY2LGYxMixmMTQsZjIwJl89',
        'jQuery1124024362308906615082_1628258931224')
    a = [ ['hk'+i[0], i[1], i[2], i[3], i[4]] for i in a]
    logger.debug('get hk stocks GangGuTong {}'.format(len(a)))
    return a


def get_hk_stocks_hsi():
    '''
    Return sorted stock item list in HSI, ordered by amount of money,
    item in returned list are [code, name, change, amount, price]
    '''
    a = _east_list_fmt('aHR0cHM6Ly81Ni5wdXNoMi5lYXN0bW9uZXkuY29tL2FwaS9xdC9jbG'+
        'lzdC9nZXQ/Y2I9alF1ZXJ5MTEyNDA3ODg4ODY4NDU5NDc5NzkyXzE2MjgyNTk1NjQ2NzE'+
        'mcG49MSZwej0xMDAwJnBvPTEmbnA9MSZ1dD1iZDFkOWRkYjA0MDg5NzAwY2Y5YzI3ZjZm'+
        'NzQyNjI4MSZmbHR0PTImaW52dD0yJmZpZD1mNiZmcz1iOkRMTUswMTQxJmZpZWxkcz1mM'+
        'yxmNixmMTIsZjE0LGYyMCZfPQ==',
        'jQuery112407888868459479792_1628259564671')
    a = [ ['hk'+i[0], i[1], i[2], i[3], i[4]] for i in a]
    logger.debug('get hk stocks HSI {}'.format(len(a)))
    return a







# -*- coding: utf-8 -*-

import time
import random
import logging
import httpx
import numpy as np
import pandas as pd
import uuid

def setup_logger():
    logger = logging.getLogger('rquote')
    if not logger.handlers:
        logger.setLevel(logging.INFO)
        file_handler = logging.FileHandler('/tmp/rquote.log')
        
        formatter = logging.Formatter('%(asctime)-15s:%(lineno)s %(message)s')
        file_handler.setFormatter(formatter)
        
        logger.addHandler(file_handler)
        logger.addHandler(logging.StreamHandler())
    
    return logger

logger = setup_logger()


class WebUtils:
    @staticmethod
    def ua():
        ua_list = [
            'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.101',
            'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/38.0.2125.122',
            'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.71',
            'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.95',
            'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.71',
            'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQDownload 732; .NET4.0C; .NET4.0E)',
            'Mozilla/5.0 (Windows NT 5.1; U; en; rv:1.8.1) Gecko/20061208 Firefox/2.0.0 Opera 9.50',
            'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:34.0) Gecko/20100101 Firefox/34.0',
            'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_3) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.5 Safari/534.55.3',
            'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.114 Safari/537.36',
            'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.114 Safari/521.61'
        ]
        return random.choice(ua_list)

    @classmethod
    def headers(cls):
        header = {
            'referer': str(uuid.uuid4()),
            'user-agent': cls.ua()
            }
        return header

    @classmethod
    def http_get(cls, url, headers, method, proxy=None):
        '''
        request.get() wrapper
        '''
        headers['user-agent'] = cls.ua
        try:
            r = httpx.get(url, allow_redirects=True)
        except Exception as e:
            logger.error('Fetch url {} err: {}'.format(url, e))
            return None
        if r:
            if method == 'text':
                return r.text
            elif method == 'content':
                return r.content

    @classmethod
    def test_proxy(cls, proxy: str):
        '''
        proxy format 'ip:port'
        test baidu.com for cn
        # test google.com for non-cn (not effective due to DNS hijacking)
        '''
        try:
            with httpx.Client(proxies=proxy) as client:
                r = client.get('https://baidu.com', timeout=2)
                if r.ok:
                    return 1
                else:
                    return 0
        except Exception as e:
            logger.info(f'test proxy {proxy} negative')
            return 0
        if r.ok:
            logger.info(f'test proxy {proxy} positive')
            return 1


class BasicFactors:

    @staticmethod
    def break_rise(d) -> float:
        if d.open[-1] / d.close[-2] > 1.002 and d.close[-1] > d.open[-1]:
            return round((d.open[-1] - d.close[-2]) / d.close[-2], 2)
        else:
            return 0

    @staticmethod
    def min_resist(d) -> float:
        sup, pre, pcur = 0, 0, d.close[-1]
        for i in d.iterrows():
            p = (i[1].open + i[1].close) / 2
            if p > pcur:
                pre += i[1].vol
            if p < pcur:
                sup += i[1].vol
        minres = (sup - pre) / (sup + pre)
        if abs(minres - 1) < .01 and d.close[-2] < max(d.close[:-2]):
            minres += .2
        minres = round(minres, 2)
        return minres

    @staticmethod
    def vol_extreme(d):
        d = d.vol
        v60max = d.rolling(60).max()
        v60min = d.rolling(60).min()
        # any in last 3days
        for i in range(1, 3):
            if d[-i] > v60max[-i - 1]:
                return round(d[-i] / v60max[-i - 1], 2)
            if d[-i] < v60min[-i - 1]:
                return round(-d[-i] / v60min[-i - 1], 2)
            else:
                return 0

    @staticmethod
    def bias_rate_over_ma60(d):
        r60 = d.close - d.close.rolling(60).mean()
        if r60[-1] > 0:
            return round(r60[-1] / r60.rolling(60).max()[-1], 2)
        else:
            return round(-r60[-1] / r60.rolling(60).min()[-1], 2)

    @staticmethod
    def op_ma(d) -> float:
        ''' op: ma score'''
        if len(d) < 22:
            return
        d['mv5'] = d.close.rolling(5).mean()
        d['mv10'] = d.close.rolling(10).mean()
        d['mv20'] = d.close.rolling(20).mean()
        d['mv60'] = d.close.rolling(60).mean()

        def ma20(d):
            ret = 0
            # .2 for over ma60
            if d.close[-1] > d.mv60[-1]:
                ret += 0.2
            # .2 for all upwards ma's
            if (d.mv5[-1] > d.mv5[-2] and d.mv10[-1] >
                    d.mv10[-2] and d.mv20[-1] > d.mv20[-2]):
                ret += 0.2
                for j in range(1, 3):
                    if not (d.close[-j] > d.mv5[-j] and d.close[-j]
                            > d.mv10[-j] and d.close[-j] > d.mv20[-j]):
                        return ret
                for j in range(3, 5):
                    if (d.close[-j] > d.mv5[-j] and d.close[-j] >
                            d.mv10[-j] and d.close[-j] > d.mv20[-j]):
                        return ret
                # .2 for just rush over ma's (fresh score)
                ret += 0.2
            return ret
        return ma20(d)

    @staticmethod
    def op_cnt(d, cont_min=3) -> (int):
        ''' op: count continous bulling days over index'''
        d.index = pd.DatetimeIndex(d.index)
        td = (d.p_change_on_sh.rolling(cont_min).min() > 0).astype(int) * \
            (d.p_change.rolling(cont_min).min() > 0).astype(int)
        ret = 0 if td[-1] <= 0 else td[-1]
        # is_first_day = True if td[-2] <= 0 else False
        return ret


class hget:
    '''
    class version request.get wrapper
    '''
    def __init__(self, url, *args, **kwargs):
        self.url = url
        try:
            r = httpx.get(
                self.url, follow_redirects=True, headers=WebUtils.headers(),
                *args, **kwargs)
            self.text = r.text
            self.content = r.content
        except Exception as e:
            logger.error(f'fetch {self.url} err: {e}')
            self.text = ''
            self.content = b''




from os import path as os_path
from setuptools import setup

this_directory = os_path.abspath(os_path.dirname(__file__))

def read_file(filename):
    with open(os_path.join(this_directory, filename), encoding='utf-8') as f:
        long_description = f.read()
    return long_description

def read_requirements(filename):
    return [line.strip() for line in read_file(filename).splitlines()
            if not line.startswith('#')]

setup(
    name='rquote',
    python_requires='>=3.4.0',
    version='0.2.6',
    description='Mostly day quotes of cn/hk/us/fund/future markets, side with quote list fetch',
    long_description=read_file('README.md'),
    long_description_content_type="text/markdown",
    author="Roizhao",
    author_email='roizhao@gmail.com',
    url='https://github.com/kids/rquote',
    packages=[
        'rquote'
    ],
    install_requires=read_requirements('requirements.txt'),
    include_package_data=True,
    license="MIT",
    keywords=['quotes', 'stock', 'rquote'],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
)
